/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area.actions;

import net.wordrider.area.RiderArea;
import net.wordrider.area.RiderDocument;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.core.MainAppFrame;
import net.wordrider.dialogs.PictureDialog;
import net.wordrider.ti89.InvalidDataTypeException;
import net.wordrider.ti89.NotSupportedFileException;
import net.wordrider.ti89.TIImageDecoder;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Utilities;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

/**
 * @author Vity
 */
public final class InsertPictureAction extends TextAreaAction {
    private static final InsertPictureAction instance = new InsertPictureAction();
    private static final String key = "lastPictureFolder";
    private static final String CODE = "InsertPictureAction";

    public static final InsertPictureAction getInstance() {
        return instance;
    }

    private InsertPictureAction() {
        super(CODE, KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK), "ins_pic_v2.gif");
    }

    public final void actionPerformed(final ActionEvent e) {
        final RiderArea area = getRiderArea(e);
        if (area != null) {
            final File currentDirectory = new File(AppPrefs.getProperty(key, ""));
            final PictureDialog fileDialog = currentDirectory.exists() ? new PictureDialog(currentDirectory) : new PictureDialog();
            final MainAppFrame frame = MainApp.getInstance().getMainAppFrame();
//            fileDialog.setFileFilter((FileFilter) new MyFileFilter());
//            fileDialog.showOpenDialog(frame);
            if (fileDialog.showInsertDialog(frame) != PictureDialog.APPROVE_OPTION)
                return;
            final File f = fileDialog.getSelectedFile();
            if (f == null)
                return;
            AppPrefs.storeProperty(key, f.getAbsolutePath());
            //currentDirectory = new File(f.getAbsolutePath());
            final int caretPosition = area.getCaretPosition();
            try {
                final boolean rowstart = Utilities.getRowStart(area, caretPosition) == caretPosition;
                if (!rowstart) {
                    area.getDocument().insertString(area.getCaretPosition(), "\n", null);
                }
                final TIImageDecoder ti = new TIImageDecoder();
                try {
                    if (!ti.openFromFile(f)) {
                        JOptionPane.showMessageDialog(area, "Image was loaded but the file might be corrupted (checksum mismatch)", "Warning", JOptionPane.WARNING_MESSAGE);
                    }
                } catch (InvalidDataTypeException ex) {
                    Utils.log(Utils.WARN_LOG, ex.getMessage());
                    JOptionPane.showMessageDialog(area, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                } catch (NotSupportedFileException ex) {
                    Utils.log(Utils.WARN_LOG, ex.getMessage());
                    JOptionPane.showMessageDialog(area, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                } catch (IOException ex) {
                    Utils.processException(ex);
                    JOptionPane.showMessageDialog(area, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
                    return;
                }
                final RiderDocument doc = area.getDoc();
                doc.removeBookmark(area.getDoc().getParagraphElement(caretPosition));
                doc.insertPicture(caretPosition, Toolkit.getDefaultToolkit().createImage(ti));
                doc.insertString(caretPosition + 1, "\n", null);
            } catch (BadLocationException ex) {
                Utils.processException(ex);
            }
        }
    }

}
