/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/**
 * 
 * @author Vity 
 */
package net.wordrider.core;

import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Utils;


public final class MainApp {
    private MainAppFrame mainAppFrame;
    private static final MainApp ourInstance = new MainApp();
    private static final String JAVA_VERSION = "java.version";


    public static final void makeProgress() {
        Utils.log(Utils.INFO_LOG, "Loading ... please wait");
    }


    public static MainApp getInstance() {
        return ourInstance;
    }

    private MainApp() {
    }

    public static void main(final String[] args) {
        if (args.length >= 1) {
            String arg = args[0];
            if (arg.endsWith("?") || arg.endsWith("help")) {
                System.out.println(Consts.APPVERSION);
                System.out.println("created by VitySoft(c)2004 /" + Consts.WEBURL + "/");
                System.out.println("usage: wordrider [-help|-debug] [file]");
                System.out.println("commands: -help  - to view this message");
                System.out.println("          -debug - enable debug log level");
                System.out.println("min. Java version required : 1.4");
                return;
            } else if (arg.endsWith("debug")) {
                Consts.LOG_SEVERITY = Utils.DEBUG_LOG;
                arg = (args.length > 1) ? args[1] : null;
            }
            getInstance().start(arg);
        } else
            getInstance().start(null);
    }


    private void start(final String openFile) {
        final String javaVersion = System.getProperty(JAVA_VERSION);
        if (javaVersion == null) {
            System.err.println("Java version was not specified. Fatal error. Cannot continue.");
            return;
        }
        final float jv = Float.parseFloat(javaVersion.substring(0, 3));
        if (!(jv > 1.39)) {
            System.err.println("Whooops. Application requires a 1.4 version or later of the Java platform.");
            System.err.println("You are running a " + javaVersion + "version of the Java platform.");
            System.err.println("Please upgrade to a newer version. Read a readme text file(if you find any ;-) for available URLs.");
            System.exit(-1);
        }

        getMainAppFrame().init(openFile);
    }

    public final MainAppFrame getMainAppFrame() {
        if (mainAppFrame == null) {
            return mainAppFrame = new MainAppFrame();
        }
        return mainAppFrame;
    }


}
