/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core;

import net.wordrider.area.RiderStyles;
import net.wordrider.core.actions.ExitAction;
import net.wordrider.core.actions.OpenFileAction;
import net.wordrider.core.managers.ManagerDirector;
import net.wordrider.gui.LookAndFeels;
import net.wordrider.plugintools.BreakpointList;
import net.wordrider.plugintools.CharacterList;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.File;

/**
 * @author Vity
 */
public final class MainAppFrame extends JFrame {
    private JPanel rootPanel;
    private ManagerDirector mainPanelManager;


    static {
        LookAndFeels.loadLookAndFeelSettings();
    }

    public MainAppFrame() throws HeadlessException {
        super();    //call to super
        final ImageIcon icon = Swinger.getIcon("011.png");

        if (icon != null)
            this.setIconImage(icon.getImage());
        
        final DisplayMode mode = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDisplayMode();
        this.setSize(mode.getWidth(), mode.getHeight());
        this.setTitle(Consts.APPVERSION + (Utils.isDefinedForLog(Utils.DEBUG_LOG) ? " debug" : ""));
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setContentPane(getMainPanel());
    }

    protected final JRootPane createRootPane() {
        final ActionListener actionListener = new ActionListener() {
            public void actionPerformed(final ActionEvent actionEvent) {
                mainPanelManager.getStatusbarManager().specialKeyStatusChanged();
            }
        };
        final JRootPane rootPane = new JRootPane();
        KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_NUM_LOCK, 0);
        rootPane.registerKeyboardAction(actionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
        stroke = KeyStroke.getKeyStroke(KeyEvent.VK_CAPS_LOCK, 0);
        rootPane.registerKeyboardAction(actionListener, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);
        return rootPane;
    }

    public final void init(final String openFile) {
        getManagerDirector();
        if (openFile != null) {
            OpenFileAction.getInstance().open(new File(openFile));
        } else
            mainPanelManager.getAreaManager().openFileInstance();
        mainPanelManager.getPluginToolsManager().addPluginTool(new CharacterList());
        mainPanelManager.getPluginToolsManager().addPluginTool(new BreakpointList());
        mainPanelManager.getAreaManager().grabActiveFocus();
        this.setExtendedState(JFrame.MAXIMIZED_BOTH);
        this.setVisible(true);        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                final String[] fileChooser = new String[]{"FileChooser.lookInLabelText", "FileChooser.saveInLabelText", "FileChooser.fileNameLabelText", "FileChooser.filesOfTypeLabelMnemonic", "FileChooser.filesOfTypeLabelText", "FileChooser.upFolderToolTipText", "FileChooser.upFolderAccessibleName", "FileChooser.homeFolderToolTipText", "FileChooser.homeFolderAccessibleName", "FileChooser.newFolderToolTipText", "FileChooser.newFolderAccessibleName", "FileChooser.listViewButtonToolTipText", "FileChooser.listViewButtonAccessibleName", "FileChooser.detailsViewButtonToolTipText", "FileChooser.detailsViewButtonAccessibleName", "FileChooser.fileNameHeaderText", "FileChooser.fileSizeHeaderText", "FileChooser.fileTypeHeaderText", "FileChooser.fileDateHeaderText", "FileChooser.fileAttrHeaderText", "FileChooser.cancelButtonText","FileChooser.cancelButtonToolTipText","FileChooser.deleteFileButtonText","FileChooser.filesLabelText","FileChooser.foldersLabelText","FileChooser.newFolderButtonText","FileChooser.openButtonText","FileChooser.openButtonToolTipText","FileChooser.openDialogTitleText","FileChooser.pathLabelText","FileChooser.renameFileButtonText","FileChooser.saveButtonText","FileChooser.saveButtonToolTipText","FileChooser.saveDialogTitleText"};
                String value;
                for (int i = 0; i < fileChooser.length; ++i) {
                    value = fileChooser[i];
                    UIManager.put(value, AppPrefs.getLabel(value));
                }
            }
        });
        if (!RiderStyles.isCorrectFont()) {
            Swinger.showErrorDialog(this, AppPrefs.getLabel("message.error.fontloadfailed", new Object[]{RiderStyles.FONT_NAME, RiderStyles.FONT_FAMILY}));
        }
    }


    private JPanel getMainPanel() {
        if (rootPanel == null) {
            rootPanel = new JPanel();
            rootPanel.setLayout(new BorderLayout());
            //create panels
        }
        return rootPanel;
    }

    public final ManagerDirector getManagerDirector() {
        if (mainPanelManager == null) {
            mainPanelManager = new ManagerDirector(this, rootPanel);
        }
        return mainPanelManager;
    }


    protected final void processWindowEvent(final WindowEvent e) {
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            ExitAction.getInstance().actionPerformed(null);//check for opened files
            //System.exit(0);
        }
    }


}
