/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.actions;

import net.wordrider.core.AppPrefs;
import net.wordrider.gui.LookAndFeels;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import java.awt.event.ActionEvent;

/**
 * @author Vity
 */
public class ChangeLookAndFeelAction extends CoreAction {
    private final static String CODE = "ChangeLookAndFeelAction";
    private final String className, theme;

    public ChangeLookAndFeelAction(String className, String name, String theme) {
        super(CODE, name, new Integer('\0'), null);
        this.className = className;
        this.theme = theme;
    }

    public void actionPerformed(final ActionEvent e) {
        boolean succesful;
        try {
            succesful = LookAndFeels.loadLookAndFeel(className, theme, true);
            LookAndFeels.storeSelectedLaF(className, theme);

        } catch (Exception ex) {
            Utils.processException(ex);
            succesful = false;
        }
        if (succesful)
            Swinger.showInformationDialog(getMainFrame(), AppPrefs.getLabel("ChangeLookAndFeelAction.set"));
        else 
            Swinger.showErrorDialog(getMainFrame(),AppPrefs.getLabel("ChangeLookAndFeelAction.failed"));
    }
}
