/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.actions;

import net.wordrider.core.AppPrefs;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.dialogs.SaveSettingsDialog;
import net.wordrider.ti89.TITextFileInfo;
import net.wordrider.ti89.TITextFileWriter;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.plaf.FileChooserUI;
import javax.swing.plaf.basic.BasicFileChooserUI;
import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

/**
 * @author Vity
 */
public class SaveAsFileAction extends CoreAction {
    private static final SaveAsFileAction instance = new SaveAsFileAction();
    //  private static File currentDirectory;
    private static final String CODE = "SaveAsFileAction";
    private static final String LASTFOLDER_KEY = OpenFileAction.LASTOPENFOLDER_KEY;

    public static SaveAsFileAction getInstance() {
        return instance;
    }

    private SaveAsFileAction() {
        super(CODE, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK), "save_as.gif");
    }

    boolean runSaveProcess(final JFrame frame, final File f, final IFileInstance instance, boolean showInfoWindow) {
        try {
            final net.wordrider.core.actions.TextRiderProcess process = new net.wordrider.core.actions.TextRiderProcess();
            final String s = process.getTxtRiderTextForDocument((StyledDocument) instance.getTextComponent().getDocument(), instance.getFileInfo(), true);
            Utils.log(Utils.DEBUG_LOG, s);
            final TITextFileWriter file = new TITextFileWriter();
            file.setFileInfo(instance.getFileInfo());
            file.setTextContent(s);
            file.saveToFile(f);
            instance.setFile(f);
        } catch (BadLocationException ex) {
            Swinger.showErrorDialog(frame, AppPrefs.getLabel("message.error.Fwriting", ex.getMessage()));
            Utils.processException(ex);
            return false;
        } catch (IOException ex) {
            Utils.log(Utils.ERROR_LOG, ex.getMessage());
            Swinger.showErrorDialog(frame, AppPrefs.getLabel("message.error.saving", ex.getMessage()));
            return false;
        }
        // == null - no info window
        if (frame != null && showInfoWindow)
            Swinger.showInformationDialog(frame, AppPrefs.getLabel("message.information.savedSuccess", f.getAbsolutePath()));
        return true;
    }

    boolean saveAsProcess(final ActionEvent e, boolean showInfoWindow) {
        final JFrame frame = getMainFrame();
        IFileInstance instance = getAreaManager().getActiveFileInstance();
        if (instance == null)
            return false;
        final SaveSettingsDialog saveSettingsDialog = new SaveSettingsDialog(frame, instance.getFileInfo());
        final TITextFileInfo info = saveSettingsDialog.getResult();
        if (info == null)
            return false;
        else
            instance.setFileInfo(info);
        final File currentDirectory = new File(AppPrefs.getProperty(LASTFOLDER_KEY, ""));
        final JFileChooser fileDialog = new JFileChooser(currentDirectory);
        fileDialog.setDialogTitle(AppPrefs.getLabel(CODE + ".dialog.title"));
        final FileChooserUI chooserUI = fileDialog.getUI();
        if (chooserUI instanceof BasicFileChooserUI)
            ((BasicFileChooserUI) chooserUI).setFileName(info.getVarName());
        fileDialog.setFileFilter(instance.getFileFilter());
        if (fileDialog.showSaveDialog(frame) != JFileChooser.APPROVE_OPTION)
            return false;
        final FileFilter filter = fileDialog.getFileFilter();
        File f = fileDialog.getSelectedFile();
        if (f == null)
            return false;
        //final String activeExt;
        //fill extension for lazy user
        if (filter instanceof IFileType && (Utils.getExtension(f)) == null) {
            final String extension = ((IFileType) filter).getExtension();
            //if (!activeExt.equalsIgnoreCase(extension))
            f = new File(f.getPath().concat(".").concat(extension));
        }
        if (f.exists()) {
            final int result = Swinger.getChoice(frame, AppPrefs.getLabel("message.confirm.overwrite"));
            if (result == Swinger.RESULT_NO)
                return false;    //?
        }
        AppPrefs.storeProperty(LASTFOLDER_KEY, f.getAbsolutePath());
        return runSaveProcess(frame, f, instance, showInfoWindow);
    }

    public void actionPerformed(ActionEvent e) {
        saveAsProcess(e, true);
    }

}
