/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.area.RiderArea;
import net.wordrider.area.actions.*;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.actions.*;
import net.wordrider.core.managers.interfaces.IAreaChangeListener;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.dialogs.JButtonGroup;
import net.wordrider.gui.LookAndFeels;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Swinger;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.util.Collection;
import java.util.Iterator;
import java.util.Properties;

/**
 * @author Vity
 */
public final class MenuManager implements IAreaChangeListener {
    //private final ManagerDirector director;
    private RiderArea editor;
    private final JMenuBar menuBar = new JMenuBar();
    private JMenu fileMenu, editMenu, insertMenu, formatMenu, paragraphMenu, viewMenu, wordRiderMenu, widthViewMenu, recentMenu;
    private JMenuItem undoItem, redoItem;
    private JMenuItem cutItem, copyItem, pasteItem, selectAllItem, findItem;
    private JMenuItem separatorSingleItem, separatorDoubleItem;//, pictureItem, bookmarkItem;
//    private JMenuItem size1Item, size2Item, size3Item, superScriptItem, underlineSingleItem, underlineDottedItem, strikeoutItem, invertItem, vectorItem, wordWrapItem;
//    private JMenuItem alignLeftItem, alignRightItem, alignCenterItem, margin10Item, margin20Item, margin30Item, mathItem;
//    private JMenuItem newFileItem, openFileItem, saveAsFileItem, saveFileItem, closeItem, closeAllItem, exitItem;
    private JCheckBoxMenuItem viewToolbarItem, viewStatusBarItem, viewAntialiased;
    private JRadioButtonMenuItem fullViewItem, ti89ViewItem, ti92ViewItem;
    private static final String languagesListFile = "languages.properties";
    private static final String languageNonameIcon = "blank.gif";

    public MenuManager(final ManagerDirector director) {
        super();    //call to super
        //this.director = director;
        director.getMainFrame().setJMenuBar(menuBar);

        menuBar.add(getFileMenu());
        menuBar.add(getEditMenu());
        menuBar.add(getFormatMenu());
        menuBar.add(getParagraphMenu());
        menuBar.add(getInsertMenu());
        menuBar.add(getViewMenu());
        menuBar.add(getWordRiderMenu());

    }

    private static final JMenu initMenu(final String code) {
        JMenu menu = new JMenu(AppPrefs.getLabel("menu." + code));
        menu.setMnemonic(AppPrefs.getMnemonic("menu." + code));
        return menu;
    }

    private Component getViewMenu() {
        if (viewMenu == null) {
            viewMenu = initMenu("view");
            Action action = ToggleToolbarAction.getInstance(viewToolbarItem = new JCheckBoxMenuItem());
            viewToolbarItem.setAction(action);
            viewMenu.add(viewToolbarItem);
            action = ToggleStatusBarAction.getInstance(viewStatusBarItem = new JCheckBoxMenuItem());
            viewStatusBarItem.setAction(action);
            viewMenu.add(viewStatusBarItem);
            viewMenu.addSeparator();
            action = ToggleAntialisingAreaAction.getInstance(viewAntialiased = new JCheckBoxMenuItem());
            viewMenu.add(viewAntialiased);
            viewAntialiased.setAction(action);
            viewMenu.addSeparator();
            viewMenu.add(widthViewMenu = initMenu("view.emulation"));
            JButtonGroup buttonGroup = new JButtonGroup();
            buttonGroup.add(fullViewItem = new JRadioButtonMenuItem(ToggleFullViewAreaAction.getInstance()));
            buttonGroup.add(ti89ViewItem = new JRadioButtonMenuItem(Toggle89ViewAreaAction.getInstance()));
            buttonGroup.add(ti92ViewItem = new JRadioButtonMenuItem(Toggle92ViewAreaAction.getInstance()));
            fullViewItem.setSelected(true);
            widthViewMenu.add(fullViewItem);
            widthViewMenu.add(ti89ViewItem);
            widthViewMenu.add(ti92ViewItem);
        }
        return viewMenu;
    }

    private Component getWordRiderMenu() {
        if (wordRiderMenu == null) {
            wordRiderMenu = initMenu("wordrider");
            JMenu language = initMenu("ChangeLanguageAction");
            JMenu lafMenu = initMenu("ChangeLookAndFeelAction");
            wordRiderMenu.add(language);
            wordRiderMenu.add(lafMenu);
            initLanguageMenu(language);
            lafMenu.setMnemonic(AppPrefs.getMnemonic("ChangeLookAndFeelAction"));
            language.setMnemonic(AppPrefs.getMnemonic("ChangeLanguageAction"));
            initLookAndFeelMenu(lafMenu);
            wordRiderMenu.addSeparator();
            wordRiderMenu.add(new JMenuItem(CheckForNewVersion.getInstance()));
            wordRiderMenu.addSeparator();
            wordRiderMenu.add(new JMenuItem(RunGCAction.getInstance()));
            wordRiderMenu.addSeparator();
            wordRiderMenu.add(new JMenuItem(ShowAboutAction.getInstance()));
        }
        return wordRiderMenu;
    }

    private void initLookAndFeelMenu(final JMenu lafMenu) {
        final Properties properties = AppPrefs.loadProperties(Consts.LAFSDIRFILE, true);
        JRadioButtonMenuItem menuItem;
        final JButtonGroup buttonGroup = new JButtonGroup();
        int counter = -1;
        final String namePostfix = ".name", themePostfix = ".theme";
        final String lafPrefix = "laf";
        String className, lafID, theme;
        while ((className = properties.getProperty(lafID = (lafPrefix + ++counter))) != null) {
            if (LookAndFeels.isPresent(className) != null) {
                theme = properties.getProperty(lafID + themePostfix, null);
                if (theme == null || (theme != null && LookAndFeels.isPresent(theme) != null)) {
                    buttonGroup.add(menuItem = new JRadioButtonMenuItem(new ChangeLookAndFeelAction(className, properties.getProperty(lafID + namePostfix, className), theme)));
                    lafMenu.add(menuItem);
                    menuItem.setSelected(LookAndFeels.isSelectedLaF(className, theme));
                }
            }
        }
    }

    private void initLanguageMenu(final JMenu langMenu) {
        langMenu.setIcon(Swinger.getIcon(languageNonameIcon));
        final Properties properties = AppPrefs.loadProperties(Consts.LANGUAGESDIR + languagesListFile, true);
        int counter = -1;
        final String lngPostfix = "language", lngNamePostfix = ".name", lngMnemonicPostfix = ".mnemonic", lngIconPostfix = ".icon", selectedLanguage = AppPrefs.getSelectedLanguageCode();
        String lngCode, lngItem;
        Integer mnemonic;
        JRadioButtonMenuItem menuItem;
        final JButtonGroup buttonGroup = new JButtonGroup();
        while ((lngCode = properties.getProperty(lngItem = (lngPostfix + ++counter))) != null) {
            mnemonic = new Integer(properties.getProperty(lngItem + lngMnemonicPostfix, "\0").charAt(0));
            buttonGroup.add(menuItem = new JRadioButtonMenuItem(new ChangeLanguageAction(lngCode, properties.getProperty(lngItem + lngNamePostfix), mnemonic, properties.getProperty(lngItem + lngIconPostfix, languageNonameIcon))));
            langMenu.add(menuItem);
            menuItem.setSelected(selectedLanguage.equals(lngCode));
        }
    }

    private Component getParagraphMenu() {
        if (paragraphMenu == null) {
            paragraphMenu = initMenu("paragraph");
            paragraphMenu.add(new JMenuItem(AligmentLeftAction.getInstance()));
            paragraphMenu.add(new JMenuItem(AligmentRightAction.getInstance()));
            paragraphMenu.add(new JMenuItem(AligmentCenterAction.getInstance()));
            paragraphMenu.addSeparator();
            paragraphMenu.add(new JMenuItem(Margin10Action.getInstance()));
            paragraphMenu.add(new JMenuItem(Margin20Action.getInstance()));
            paragraphMenu.add(new JMenuItem(Margin30Action.getInstance()));
            paragraphMenu.addSeparator();
            paragraphMenu.add(new JMenuItem(ChangeMathStyleAction.getInstance()));

        }
        return paragraphMenu;
    }

    private Component getFormatMenu() {
        if (formatMenu == null) {
            formatMenu = initMenu("format");
            formatMenu.add(new JMenuItem(ChangeFontSizeStyle1Action.getInstance()));
            formatMenu.add(new JMenuItem(ChangeFontSizeStyle2Action.getInstance()));
            formatMenu.add(new JMenuItem(ChangeFontSizeStyle3Action.getInstance()));
            formatMenu.addSeparator();
            formatMenu.add(new JMenuItem(ChangeExponentStyleAction.getInstance()));
            formatMenu.addSeparator();
            formatMenu.add(new JMenuItem(ChangeUnderlineStyleAction.getInstance()));
            formatMenu.add(new JMenuItem(ChangeDottedStyleAction.getInstance()));
            formatMenu.add(new JMenuItem(ChangeStrikedStyleAction.getInstance()));
            formatMenu.addSeparator();
            formatMenu.add(new JMenuItem(ChangeInvertStyleAction.getInstance()));
            formatMenu.add(new JMenuItem(ChangeVectorStyleAction.getInstance()));
            formatMenu.add(new JMenuItem(ChangeWordWrapStyleAction.getInstance()));
        }
        return formatMenu;
    }


    private Component getInsertMenu() {
        if (insertMenu == null) {
            insertMenu = initMenu("insert");
            insertMenu.add(new JMenuItem(InsertPictureAction.getInstance()));
            insertMenu.addSeparator();
            separatorSingleItem = new JMenuItem(InsertSeparatorSingleAction.getInstance());
            insertMenu.add(separatorSingleItem);
            separatorDoubleItem = new JMenuItem(InsertSeparatorDoubleAction.getInstance());
            insertMenu.add(separatorDoubleItem);
            insertMenu.addSeparator();
            insertMenu.add(new JMenuItem(UpdateBreakpointAction.getInstance()));
        }
        return insertMenu;
    }

    private Component getFileMenu() {
        if (fileMenu == null) {
            fileMenu = initMenu("file");
            fileMenu.add(new JMenuItem(CreateNewFileAction.getInstance()));
            fileMenu.add(new JMenuItem(OpenFileAction.getInstance()));
            fileMenu.addSeparator();
            fileMenu.add(new JMenuItem(SaveFileAction.getInstance()));
            fileMenu.add(new JMenuItem(SaveAsFileAction.getInstance()));
            fileMenu.addSeparator();
            fileMenu.add(new JMenuItem(CloseActiveAction.getInstance()));
            fileMenu.add(new JMenuItem(CloseAllAction.getInstance()));
            fileMenu.addSeparator();
            fileMenu.add(recentMenu = initMenu("file.recentFiles"));
            recentMenu.setEnabled(false);
            fileMenu.addSeparator();
            fileMenu.add(new JMenuItem(ExitAction.getInstance()));
        }
        return fileMenu;
    }

    private Component getEditMenu() {
        if (editMenu == null) {
            editMenu = initMenu("edit");
            undoItem = new JMenuItem(UndoAction.getInstance());
            redoItem = new JMenuItem(RedoAction.getInstance());
            editMenu.add(undoItem);
            editMenu.add(redoItem);
            editMenu.add(new JSeparator());
            cutItem = new JMenuItem(CutAction.getInstance());
            editMenu.add(cutItem);
            copyItem = new JMenuItem(CopyAction.getInstance());
            editMenu.add(copyItem);
            pasteItem = new JMenuItem(PasteAction.getInstance());
            editMenu.add(pasteItem);
            selectAllItem = new JMenuItem(SelectAllAction.getInstance());
            editMenu.add(selectAllItem);
            editMenu.addSeparator();
            findItem = new JMenuItem(ShowFindReplaceDialogAction.getInstance());
            editMenu.add(findItem);
        }
        return editMenu;
    }


//    private void enableActions(final JTextComponent component) {
//        //parapraph
//
//
//    }

//    private final static class CheckTextStatusListener implements MenuListener {
//
//        public void menuCanceled(MenuEvent e) {
//            menuDeselected(e);
//        }
//
//        public void menuSelected(MenuEvent e) {
//            final Object source =
//                    MainApp.getInstance().getMainAppFrame().getFocusOwner();
//            if (source != null && source instanceof JTextComponent) {
//            //    final JTextComponent component = (JTextComponent) source;
//            }
//        }
//
//        public void menuDeselected(MenuEvent e) {
//        }
//    }


    public void selectedAreaChanged(final IFileInstance instance) {
        final boolean isEnabled = instance != null;
        editor = (isEnabled) ? (RiderArea) instance.getTextComponent() : null;
        RedoAction.getInstance().updateRedoState(editor);
        UndoAction.getInstance().updateUndoState(editor);

        ChangeFontSizeStyle1Action.getInstance().setEnabled(isEnabled);
        ChangeFontSizeStyle2Action.getInstance().setEnabled(isEnabled);
        ChangeFontSizeStyle3Action.getInstance().setEnabled(isEnabled);
        AligmentLeftAction.getInstance().setEnabled(isEnabled);
        AligmentCenterAction.getInstance().setEnabled(isEnabled);
        AligmentRightAction.getInstance().setEnabled(isEnabled);
        Margin10Action.getInstance().setEnabled(isEnabled);
        Margin20Action.getInstance().setEnabled(isEnabled);
        Margin30Action.getInstance().setEnabled(isEnabled);
        ChangeInvertStyleAction.getInstance().setEnabled(isEnabled);
        ChangeVectorStyleAction.getInstance().setEnabled(isEnabled);
        ChangeWordWrapStyleAction.getInstance().setEnabled(isEnabled);
        ChangeExponentStyleAction.getInstance().setEnabled(isEnabled);
        ChangeUnderlineStyleAction.getInstance().setEnabled(isEnabled);
        ChangeDottedStyleAction.getInstance().setEnabled(isEnabled);
        ChangeStrikedStyleAction.getInstance().setEnabled(isEnabled);
        UpdateBreakpointAction.getInstance().setEnabled(isEnabled);
        InsertPictureAction.getInstance().setEnabled(isEnabled);
        InsertSeparatorDoubleAction.getInstance().setEnabled(isEnabled);
        InsertSeparatorSingleAction.getInstance().setEnabled(isEnabled);
        ChangeMathStyleAction.getInstance().setEnabled(isEnabled);
        //paragraphMenu.getActionMap().
        //paragraphMenu.setEnabled(isEnabled);
//        insertMenu.setEnabled(isEnabled);
//        formatMenu.setEnabled(isEnabled);
        if (editor != null) {
            final boolean isSelectedText = editor.getSelectionStart() != editor.getSelectionEnd();
            CopyAction.getInstance().setEnabled(isSelectedText && editor.isEditable());
            CutAction.getInstance().setEnabled(isSelectedText && editor.isEditable());
            PasteAction.getInstance().setEnabled(editor.isEditable());
            switch (editor.getViewBorder()) {
                case RiderArea.FULLVIEWBORDER:
                    fullViewItem.setSelected(true);
                    break;
                case RiderArea.TI89VIEWBORDER:
                    ti89ViewItem.setSelected(true);
                    break;
                case RiderArea.TI92VIEWBORDER:
                    ti92ViewItem.setSelected(true);
                    break;
            }
        } else {
            CopyAction.getInstance().setEnabled(isEnabled);
            CutAction.getInstance().setEnabled(isEnabled);
            CopyAction.getInstance().setEnabled(isEnabled);
            PasteAction.getInstance().setEnabled(isEnabled);

        }
        widthViewMenu.setEnabled(isEnabled);
        SelectAllAction.getInstance().setEnabled(isEnabled);
        ShowFindReplaceDialogAction.getInstance().setEnabled(isEnabled);
        SaveFileAction.getInstance().setEnabled(isEnabled && editor.isModified());
        SaveAsFileAction.getInstance().setEnabled(isEnabled);
        CloseActiveAction.getInstance().setEnabled(isEnabled);
        CloseActiveAction.getInstance().updateStatusName(instance);
        CloseAllAction.getInstance().setEnabled(isEnabled);
    }

    public void updateRecentMenu(final Collection recentFiles){
        recentMenu.removeAll();
        int position = 0;
        File file;
        for (Iterator it = recentFiles.iterator(); it.hasNext(); position ++){
            file = (File)it.next();
            recentMenu.add(new JMenuItem(new OpenRecentFileAction(file, getMnemonicByPosition(position))));
        }
        recentMenu.setEnabled(recentFiles.size() > 0);
    }

    private char getMnemonicByPosition(final int position) {
        return (char)((position > 9) ? 'A' + position - 10 : '0' + position);
    }

}
