/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.core.managers.interfaces.IInformedTab;
import net.wordrider.core.managers.interfaces.IRiderManager;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.util.HashMap;
import java.util.Map;


/**
 * @author Vity
 */
public abstract class TabManager implements IRiderManager {
    protected final JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.SCROLL_TAB_LAYOUT);

    private static int anIDCounter;
    /**
     * LASTOPENFOLDER_KEY - component
     * value - id
     * *
     */
    protected final Map runningTabs = new HashMap(4);
    /**
     * LASTOPENFOLDER_KEY - id
     * value - fileinstance
     * *
     */
    protected final Map runningInstancesIDs = new HashMap(4);
    protected Integer activeInstanceID;

    public TabManager() {
        //InputMap map = tabbedPane.getInputMap();
        //tabbedPane.setInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW, map);
        tabbedPane.setFocusable(false);
        //tabbedPane.setF
        tabbedPane.getModel().addChangeListener(new TabListener());
    }

    final Object getActiveInstance() {
        return (activeInstanceID != null) ? runningInstancesIDs.get(activeInstanceID) : null;
    }

    public final Component getManagerComponent() {
        return tabbedPane;
    }

    final void registerNewOne(final IInformedTab informedPlugin) {
        final Integer id = nextID();
        runningInstancesIDs.put(id, informedPlugin);
        final Component managerComponent = informedPlugin.getComponent();
        runningTabs.put(managerComponent, id);
        tabbedPane.addTab(informedPlugin.getTabName(), informedPlugin.getIcon(), managerComponent, informedPlugin.getTip());
        tabbedPane.setSelectedComponent(managerComponent);
    }

    protected void deactivateInstance(final Integer anID) {
        ((IInformedTab) runningInstancesIDs.get(anID)).deactivate();
    }


    private final class TabListener implements ChangeListener {
        public final void stateChanged(final ChangeEvent e) {
            if (tabbedPane.getSelectedComponent() != null) {
                final Integer anID = (Integer) runningTabs.get(tabbedPane.getSelectedComponent());
                if (anID != null && !anID.equals(activeInstanceID)) {
                    activateInstance(anID);
                } else {
                    if (activeInstanceID != null) {
                        deactivateInstance(activeInstanceID);
                        activeInstanceID = null;
                        //MainApp.getMainApp().getMainAppFrame().setTitle();
                    }
                }
            } else
                activeInstanceID = null;
        }
    }

//    private void activateInstance(final Integer anID) {
//        activateInstance(anID, getActiveInstance());
//    }


    void activateInstance(final Integer anID) {
        //deactivating active
        if (activeInstanceID != null && runningInstancesIDs.containsKey(activeInstanceID))
            deactivateInstance(activeInstanceID);
        //activating new
        activeInstanceID = anID;
        final IInformedTab fileInstance = (IInformedTab) runningInstancesIDs.get(anID);
        fileInstance.activate();
    }
    //  MainApp.getMainApp().getMainAppFrame().setTitle();

    protected void closeSoft(final Integer anID) {
        if (anID == null || !runningInstancesIDs.containsKey(anID))
            return;
        final IInformedTab informedTab =
                (IInformedTab) runningInstancesIDs.get(anID);
        final boolean result = informedTab.closeSoft();
        if (result)
            removeInstance(anID);
    }

    protected void closeHard(final Integer anID) {
        if (anID == null || !runningInstancesIDs.containsKey(anID))
            return;
        removeInstance(anID);
    }

    private synchronized void removeInstance(final Integer anID) {
        final IInformedTab informedTab =
                (IInformedTab) runningInstancesIDs.get(anID);
        runningTabs.remove(informedTab.getComponent());
        //order!
        if (runningTabs.size() == 0)
            deactivateInstance(anID);
        runningInstancesIDs.remove(anID);
        tabbedPane.remove(informedTab.getComponent());
        tabbedPane.validate();
        activeInstanceID = null;
        if (tabbedPane.getSelectedComponent() != null)
            activateInstance((Integer) runningTabs.get(tabbedPane.getSelectedComponent()));
    }


    private static synchronized Integer nextID() {
        return new Integer(++anIDCounter);
    }

    public void getPrevTab() {
        final int tabCount = runningTabs.size();
        if (tabCount > 1) {
            int selectedIndex = tabbedPane.getSelectedIndex();
            if (selectedIndex - 1 < 0)
                selectedIndex = tabCount;
            tabbedPane.setSelectedIndex(selectedIndex - 1);
        }
    }

    public void getNextTab() {
        final int tabCount = runningTabs.size();
        if (tabCount > 1) {
            int selectedIndex = tabbedPane.getSelectedIndex();
            if (selectedIndex + 1 == tabCount)
                selectedIndex = -1;
            tabbedPane.setSelectedIndex(selectedIndex + 1);
        }
    }
}
