/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.dialogs;

import net.wordrider.core.AppPrefs;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * @author Vity
 */
public class ConnectDialog extends AppDialog {
    private JTextField inputLoginName = new JTextField();
    private JCheckBox checkProxy;
    private JButton btnCheck;
    private JButton btnCancel;
    private JPanel panelProxy;
    private JTextField inputHostName = new JTextField();
    private JCheckBox checkAuthentification;
    private JLabel labelWarning = new JLabel();
    private JTextField inputPortNumber = new JTextField();
    private JCheckBox checkSavePassword = new JCheckBox();
    private JLabel labelPassword;
    private JLabel labelLoginName;
    private JTextField inputPassword = new JPasswordField();
    private JLabel labelPortNumber;
    private JLabel labelHostName;

    public ConnectDialog(Frame owner) throws HeadlessException {
        super(owner, true);    //call to super

        try {
            init();
            initData();
            useHttpProxy(checkProxy.isSelected());
            useAuthentification(checkAuthentification.isSelected());
            this.setModal(true);
            this.setTitle(AppPrefs.getLabel("dialog.connect.title", Consts.APPVERSION));
            this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            this.pack();
            this.setSize(305, 225);
            Swinger.centerDialog(owner, this);
            this.setVisible(true);
        } catch (Exception e) {
            Utils.processException(e);
        }
    }

    private void initData() {
        checkProxy.setSelected(AppPrefs.getProperty(AppPrefs.PROXY_USE, false));
        checkAuthentification.setSelected(AppPrefs.getProperty(AppPrefs.PROXY_LOGIN, false));
        inputHostName.setText(AppPrefs.getProperty(AppPrefs.PROXY_URL, ""));
        inputPortNumber.setText(AppPrefs.getProperty(AppPrefs.PROXY_PORT, ""));
        checkSavePassword.setSelected(AppPrefs.getProperty(AppPrefs.PROXY_SAVEPASSWORD, false));
        inputLoginName.setText(AppPrefs.getProperty(AppPrefs.PROXY_USERNAME, ""));
        inputPassword.setText(Utils.generateXorString(AppPrefs.getProperty(AppPrefs.PROXY_PASSWORD, "")));
    }

    private void btnCancel_actionPerformed() {
        result = AppDialog.RESULT_CANCEL;
        cancel();
    }

    private void btnCheck_actionPerformed() {

        if (checkProxy.isSelected()) {
            if (inputHostName.getText().length() == 0) {
                Swinger.showErrorDialog((Frame)this.getParent(), AppPrefs.getLabel("dialog.connect.message.enterHostName"));
                Swinger.inputFocus(inputHostName);
                return;
            }
            if (inputPortNumber.getText().length() == 0) {
                Swinger.showErrorDialog((Frame)this.getParent(), AppPrefs.getLabel("dialog.connect.message.enterPort"));
                Swinger.inputFocus(inputPortNumber);
                return;
            }
            if (checkAuthentification.isSelected()) {
                if (inputLoginName.getText().length() == 0) {
                    Swinger.showErrorDialog((Frame)this.getParent(), AppPrefs.getLabel("dialog.connect.message.enterLogin"));
                    Swinger.inputFocus(inputLoginName);
                    return;
                }
            }

        }

        AppPrefs.storeProperty(AppPrefs.PROXY_USE, checkProxy.isSelected());
        AppPrefs.storeProperty(AppPrefs.PROXY_LOGIN, checkAuthentification.isSelected());
        AppPrefs.storeProperty(AppPrefs.PROXY_URL, inputHostName.getText());
        AppPrefs.storeProperty(AppPrefs.PROXY_PORT, inputPortNumber.getText());
        AppPrefs.storeProperty(AppPrefs.PROXY_SAVEPASSWORD, checkSavePassword.isSelected());
        AppPrefs.storeProperty(AppPrefs.PROXY_USERNAME, inputLoginName.getText());
        AppPrefs.storeProperty(AppPrefs.PROXY_PASSWORD, Utils.generateXorString(inputPassword.getText()));
        result = AppDialog.RESULT_OK;
        cancel();
    }

    private class CheckAdapter implements java.awt.event.ActionListener {

        public void actionPerformed(final ActionEvent e) {
            if (e.getSource().equals(checkProxy)) {
                useHttpProxy(checkProxy.isSelected());
            } else if (e.getSource().equals(checkAuthentification)) {
                useAuthentification(checkAuthentification.isSelected());
            }
        }
    }

    private void useAuthentification(final boolean use) {
        inputLoginName.setEnabled(use);
        inputPassword.setEnabled(use);
        checkSavePassword.setEnabled(use);
        if (use) {
            inputLoginName.setSelectionStart(0);
            inputLoginName.setSelectionEnd(inputLoginName.getText().length());
            inputLoginName.requestFocus();
        }
    }

    private void useHttpProxy(final boolean use) {
        this.getContentPane().setEnabled(use);
        checkProxy.setSelected(use);
        inputHostName.setEnabled(use);
        inputPortNumber.setEnabled(use);
        checkAuthentification.setEnabled(use);
        if (checkAuthentification.isSelected() && use)
            useAuthentification(true);
        else
            useAuthentification(false);
        if (use) {
            inputHostName.setSelectionStart(0);
            inputHostName.setSelectionEnd(inputHostName.getText().length());
            inputHostName.requestFocus();
        }
    }

    private class ActionButtonsAdapter implements ActionListener {

        public void actionPerformed(final ActionEvent e) {
            if (e.getSource().equals(btnCheck))
                btnCheck_actionPerformed();
            else
                btnCancel_actionPerformed();
        }
    }

    private void init() {
        final Container panelConnect = this.getContentPane();
        panelConnect.setLayout(new GridBagLayout());
        ActionListener checkListener = new CheckAdapter();

        Utils.addKeyActions(inputHostName);
        Utils.addKeyActions(inputLoginName);
        Utils.addKeyActions(inputPassword);
        Utils.addKeyActions(inputPortNumber);

        checkProxy = Swinger.getRiderCheckBox("dialog.connect.checkProxy");
        checkProxy.addActionListener(checkListener);
        panelConnect.add(checkProxy, new GridBagConstraints(0, 0, 3, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.VERTICAL, new Insets(1, 1, 1, 1), 0, 0));

        panelProxy = new JPanel();
        panelProxy.setLayout(new GridBagLayout());

        panelProxy.setBorder(new TitledBorder(BorderFactory.createLineBorder(Color.gray, 1), AppPrefs.getLabel("dialog.connect.labelSettings")));
        panelConnect.add(panelProxy, new GridBagConstraints(0, 1, 3, 1, 1, 1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 0, 0));

        labelHostName = new JLabel(AppPrefs.getLabel("dialog.connect.labelHostName"));
        labelHostName.setLabelFor(inputHostName);
        panelProxy.add(labelHostName, new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
        panelProxy.add(inputHostName, new GridBagConstraints(1, 0, 1, 1, 1, 0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 75, 2));

        labelPortNumber = new JLabel(AppPrefs.getLabel("dialog.connect.labelPortNumber"));
        labelPortNumber.setLabelFor(inputPortNumber);
        panelProxy.add(labelPortNumber, new GridBagConstraints(2, 0, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 4, 1, 1), 0, 0));
        panelProxy.add(inputPortNumber, new GridBagConstraints(3, 0, 1, 1, 0.1, 0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 50, 2));

        panelProxy.add(inputPassword, new GridBagConstraints(1, 3, 1, 1, 1, 0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 75, 2));

        checkAuthentification = Swinger.getRiderCheckBox("dialog.connect.checkAutentification");
        checkAuthentification.addActionListener(checkListener);
        panelProxy.add(checkAuthentification, new GridBagConstraints(0, 1, 3, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));

        labelLoginName = new JLabel(AppPrefs.getLabel("dialog.connect.labelLoginName"));
        labelLoginName.setLabelFor(inputLoginName);
        panelProxy.add(labelLoginName, new GridBagConstraints(0, 2, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
        panelProxy.add(inputLoginName, new GridBagConstraints(1, 2, 1, 1, 1, 0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 1, 1, 1), 75, 2));

        labelPassword = new JLabel(AppPrefs.getLabel("dialog.connect.labelPassword"));
        labelPassword.setLabelFor(inputPassword);
        panelProxy.add(labelPassword, new GridBagConstraints(0, 3, 1, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 2));

        checkSavePassword = Swinger.getRiderCheckBox("dialog.connect.checkSavePassword");
        panelProxy.add(checkSavePassword, new GridBagConstraints(2, 3, 2, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 2, 1, 1), 0, 4));

        labelWarning = new JLabel(AppPrefs.getLabel("dialog.connect.labelWarning"));
        panelProxy.add(labelWarning, new GridBagConstraints(0, 4, 4, 1, 0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));

        btnCheck = Swinger.getRiderButton("dialog.connect.btnCheck");
        panelConnect.add(btnCheck, new GridBagConstraints(1, 2, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 6), 20, 0));

        btnCancel = Swinger.getRiderButton("dialog.connect.btnCancel");
        panelConnect.add(btnCancel, new GridBagConstraints(2, 2, 1, 1, 1, 1, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 20, 0));

        final Dimension buttonSize = new Dimension(80, 25);

        final ActionListener actionButtonListener = new ActionButtonsAdapter();
        btnCheck.addActionListener(actionButtonListener);
        btnCancel.addActionListener(actionButtonListener);

        btnCheck.setRolloverEnabled(false);
        btnCancel.setRolloverEnabled(false);
        btnCheck.setMinimumSize(buttonSize);
        btnCheck.setPreferredSize(buttonSize);
        btnCheck.setMaximumSize(buttonSize);

        btnCancel.setMinimumSize(buttonSize);
        btnCancel.setPreferredSize(buttonSize);
        btnCancel.setMaximumSize(buttonSize);



        //panelConnect.add(progressBar, new GridBagConstraints(0, 2, 1, 1, 1.5, 1, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(1, 8, 1, 5), 60, 0));
    }
}
