package net.wordrider.plugintools;

import net.wordrider.area.RiderEditorKit;
import net.wordrider.area.RiderStyles;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Stack;

/**
 * @author Vity
 */

public final class CharacterList extends PluginTool implements MouseMotionListener, MouseListener, ItemListener {
    private final static int zoomWindowWidth = 36;
    private JTable charsTable;


    private JTable lastUsedCharsTable;
    private JTextPane editor = null;

    private JComboBox charsTypeCombo;

    private static Cursor blankCursor;
    private static boolean nowZooming = false;

    private boolean firstTime = true;

    private JWindow zoomWindow;
    private BufferedImage zoomImage = null;


    private Character activeCharacter;

    public CharacterList() {
        try {
            setFileInstance(null);
            init();
        } catch (Exception e) {
            Utils.processException(e);
        }
    }


    public final String getName() {
        return AppPrefs.getLabel("characters.title");
    }

    public String getTabName() {
        return getName();
    }

    public final Icon getIcon() {
        return Swinger.getIcon("char.gif");
    }

    public final String getTip() {
        return AppPrefs.getLabel("characters.tooltip");
    }

    public final void activate() {
        setFileInstance(MainApp.getInstance().getMainAppFrame().getManagerDirector().getAreaManager().getActiveFileInstance());
        Utils.log(Utils.DEBUG_LOG, "CharsList activated");
    }

    public final void deactivate() {
        setFileInstance(null);
        Utils.log(Utils.DEBUG_LOG, "CharsList deactivated");
    }

    private void setEditor(final JTextComponent pane) {
        this.setEnabled((this.editor = (JTextPane) pane) != null);
    }


    public void setFileInstance(final IFileInstance instance) {
        if (instance != null)
            setEditor(instance.getTextComponent());
        else
            setEditor(null);
        setEnabled(editor != null);
    }


    private CharsTableModel getCharsTableModel() {
        return ((CharsTableModel) charsTable.getModel());
    }

    private void initTable(final JTable table, final KeyListener keyListener, final TableCellRenderer renderer) {
        table.setDefaultRenderer(Object.class, renderer);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setColumnSelectionAllowed(false);
//        charsTable.getSelectionModel().addListSelectionListener(this);
        table.setRowSelectionAllowed(false);
        //charsTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);
        table.setTableHeader(null);
        table.addMouseMotionListener(this);
        table.setFont(RiderStyles.getAreaFont().deriveFont(Font.PLAIN, 13));
        table.addMouseListener(this);
        table.setRowHeight(25);
        table.addKeyListener(keyListener);
        table.setFocusCycleRoot(true);
    }


    private final void init() {
        final Toolkit tk = Toolkit.getDefaultToolkit();
        final byte bogus[] = {(byte) 0};
        blankCursor = tk.createCustomCursor(tk.createImage(bogus), new Point(0, 0), "");

        zoomWindow = new JWindow(MainApp.getInstance().getMainAppFrame()) {
            public void paint(final Graphics g) {
                g.drawImage(zoomImage, 0, 0, zoomWindow);
            }
        };
        zoomWindow.setCursor(blankCursor);
        zoomWindow.pack();

        final CellRenderer renderer = new CellRenderer();
        final KeyListener keyListener = new AddCharByKeyListener();
        charsTable = new JTable(new CharsTableModel());
        initTable(charsTable, keyListener, renderer);


        lastUsedCharsTable = new JTable(new LastUsedCharsTableModel());
        initTable(lastUsedCharsTable, keyListener, renderer);
        lastUsedCharsTable.setBorder(BorderFactory.createEtchedBorder());//createLoweredBevelBorder());

        this.setLayout(new BorderLayout());

        charsTypeCombo = new JComboBox();
        charsTypeCombo.setBorder(BorderFactory.createEmptyBorder(0, 5, 4, 5));
       // charsTypeCombo.setMinimumSize(new Dimension(100, 25));
        charsTypeCombo.setPreferredSize(new Dimension(170, 23));
        final ArrayList list = getCharsTableModel().getAvailableCharTables();
        for (final Iterator it = list.iterator(); it.hasNext();)
            charsTypeCombo.addItem(it.next());
        charsTypeCombo.addItemListener(this);
        itemStateChanged(null);


        final JPanel container = new JPanel(new BorderLayout());
        final JPanel jPanel1 = new JPanel(new BorderLayout());
        final JPanel jPanel2 = new JPanel(new BorderLayout());

        //      jPanel1.setMinimumSize(new Dimension(100, 30));
        jPanel1.setPreferredSize(new Dimension(150, 30));
        //jPanel2.setMaximumSize(new Dimension(-1,25));
        container.add(jPanel2, BorderLayout.NORTH);
        container.add(jPanel1, BorderLayout.SOUTH);

        jPanel2.setBorder(BorderFactory.createEmptyBorder(5, 4, 5, 4));
        jPanel2.add(lastUsedCharsTable, BorderLayout.CENTER);
        jPanel2.setPreferredSize(new Dimension(150, 35));

        jPanel1.add(charsTypeCombo, BorderLayout.WEST);

        final JScrollPane pane = new JScrollPane(charsTable);
        pane.setBorder(BorderFactory.createLineBorder(Color.BLACK));

        this.add(container, BorderLayout.NORTH);
        this.add(new JScrollPane(charsTable), BorderLayout.CENTER);

        this.setPreferredSize(new Dimension(180, 250));
        this.setMaximumSize(new Dimension(200, 250));
//        this.setMinimumSize(this.getPreferredSize());
        this.setVisible(true);
    }


    private final boolean firstTime() {
        return firstTime;
    }

    private final void refresh() {
        firstTime = false;
        repaint();
    }


    /// Shows (updates) the character zoom window
    private final void showZoomed(final JTable table, final int selRow, final int selColumn, final Character selCharacter) {
        if (selCharacter == null) return;
        activeCharacter = selCharacter;
        table.setCursor(blankCursor);

        if (!nowZooming) {
            if (zoomWindow.getWarningString() != null)
            /// If this is not opened as a "secure" window,
            /// it has a banner below the zoom dialog which makes it look really BAD
            /// So enlarge it by a bit
                zoomWindow.setSize(zoomWindowWidth, zoomWindowWidth);
            else
                zoomWindow.setSize(zoomWindowWidth + 1, zoomWindowWidth + 1);
        }

        /// Prepare zoomed image
        zoomImage =
                (BufferedImage) zoomWindow.createImage(zoomWindowWidth + 1,
                        zoomWindowWidth + 1);
        final Graphics2D g2 = (Graphics2D) zoomImage.getGraphics();
        final Font testFont = RiderStyles.getAreaBigFont();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setColor(Color.white);
        g2.fillRect(0, 0, zoomWindowWidth, zoomWindowWidth);
        g2.setColor(Color.black);
        g2.drawRect(0, 0, zoomWindowWidth, zoomWindowWidth);
        g2.setFont(testFont);
        final FontMetrics metrics = table.getFontMetrics(testFont);
        g2.drawString(selCharacter.toString(), (zoomWindowWidth - metrics.charWidth(selCharacter.charValue())) / 2, (zoomWindowWidth - metrics.getHeight()) / 2 + metrics.getAscent());
        g2.dispose();
        /// This is sort of redundant... since there is a paint function
        /// inside zoomWindow definition that does the drawImage.
        /// (I should be able to call just repaint() here)
        /// However, for some reason, that paint function fails to respond
        /// from second time and on; So I have to force the paint here...
        //if (nowZooming) zoomWindow.hide();
        zoomWindow.getGraphics().drawImage(zoomImage, 0, 0, this);
        final Rectangle r = table.getCellRect(selRow, selColumn, false);

        //GlyphVector gv;
        final Point canvasLoc = table.getLocationOnScreen();

        /// Calculate the zoom net.wordrider.area's location and size...
        /// Position and set size of zoom window as needed
        final int relDialogX = r.x - (zoomWindowWidth - r.width) / 2;
        final int relDialogY = r.y - (zoomWindowWidth - r.height) / 2;
        zoomWindow.setLocation(canvasLoc.x + relDialogX, canvasLoc.y + relDialogY);
        if (!nowZooming)
            zoomWindow.setVisible(true);
        nowZooming = true;
        //prevZoomChar = currMouseOverChar;

        // Windows does not repaint correctly, after
        // a zoom. Thus, we need to force the canvas
        // to repaint, but only once. After the first repaint,
        // everything stabilizes. [ABP]
        if (firstTime())
            refresh();
    }

    private static final Character getSelectedCharacter(final JTable table, final int selRow, final int selColumn) {
        return (Character) table.getModel().getValueAt(selRow, selColumn);
    }

    private static final Character getSelectedCharacter(final JTable table) {
        return (Character) table.getModel().getValueAt(table.getSelectedRow(), table.getSelectedColumn());
    }

    public final void mouseDragged(final MouseEvent e) {
        if (nowZooming) {
            final JTable table = (JTable) e.getSource();
            final int selRow = table.rowAtPoint(e.getPoint());
            final int selColumn = table.columnAtPoint(e.getPoint());

            final Character selCh = getSelectedCharacter(table, selRow, selColumn);
            if ((selCh) != null && selCh.compareTo(activeCharacter) != 0)
                showZoomed(table, selRow, selColumn, selCh);
        }
    }

    public final void mouseMoved(final MouseEvent e) {
        //implement - call to super class
    }

    public final void mouseClicked(final MouseEvent e) {
     //   mouseDragged(e);
    }

    private final JTextPane getEditor() {
        return editor;
    }

    public final void mouseEntered(final MouseEvent e) {
    }

    public final void mouseExited(final MouseEvent e) {
    }

    public final void mousePressed(final MouseEvent e) {
        final JTable table = (JTable) e.getSource();
        final int selRow = table.rowAtPoint(e.getPoint());
        final int selColumn = table.columnAtPoint(e.getPoint());
        final Character selCh = getSelectedCharacter(table, selRow, selColumn);
        showZoomed(table, selRow, selColumn, selCh);
    }

    public final void mouseReleased(final MouseEvent e) {
        if (nowZooming)
            zoomWindow.setVisible(false);
        nowZooming = false;
        final JTable table = (JTable) e.getSource();
        final Character ch = getSelectedCharacter(table, table.rowAtPoint(e.getPoint()), table.columnAtPoint(e.getPoint()));
        if (ch != null) {
            insertCharacter(ch);
            if (table.equals(charsTable))
                ((LastUsedCharsTableModel) lastUsedCharsTable.getModel()).addUsedChar(ch);
        }
        e.consume();
        table.setCursor(Cursor.getDefaultCursor());
    }

    /**
     * When in combobox changes the selection
     */
    public final void itemStateChanged(final ItemEvent e) {
        getCharsTableModel().selectCharTable(charsTypeCombo.getSelectedIndex());
    }

    private static final class CellRenderer extends DefaultTableCellRenderer {
        private static final boolean isWindows = AppPrefs.isWindows();
        /**
         * Creates a default table cell renderer.
         */
        public CellRenderer() {
            super();    //call to super
            this.setHorizontalAlignment(SwingConstants.CENTER);
            this.setHorizontalTextPosition(SwingConstants.CENTER);
        }

        public final Component getTableCellRendererComponent(final JTable table, final Object value, final boolean isSelected, final boolean hasFocus, final int row, final int column) {
            if (value != null) {
                final int charValue = ((Character) value).charValue();
                final String hotKey = (charValue < 255 && isWindows) ? "alt-0"+charValue + "  ": "";

                this.setToolTipText(hotKey + AppPrefs.getLabel("characters.unicode", Integer.toHexString(charValue)));
            }
            return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        }
    }

    private void insertCharacter(final Character ch) {
        if (editor == null)
            return;
        try {
            editor.select(editor.getCaretPosition(), editor.getCaretPosition());
            editor.getDocument().insertString(editor.getCaretPosition(), ch.toString(), editor.getInputAttributes());
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    getEditor().requestFocus();
                }
            });
        } catch (BadLocationException ex) {
            Utils.processException(ex);
            return;
        }

    }

    private final class AddCharByKeyListener implements KeyListener {
        /**
         * Invoked when a LASTOPENFOLDER_KEY has been pressed.
         * See the class description for {@link java.awt.event.KeyEvent} for a definition of
         * a LASTOPENFOLDER_KEY pressed event.
         */
        public void keyPressed(final KeyEvent e) {
            //implement - call to super class
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                final JTable table = (JTable) e.getSource();
                final Character ch = getSelectedCharacter(table);
                if (ch != null) {
                    insertCharacter(ch);
                    if (table.equals(charsTable))
                        ((LastUsedCharsTableModel) lastUsedCharsTable.getModel()).addUsedChar(ch);
                }
                e.consume();
            }

        }

        public void keyReleased(final KeyEvent e) {
            //implement - call to super class
        }

        public void keyTyped(final KeyEvent e) {
            //implement - call to super class
        }

    }


    private final static class CharsTableModel extends AbstractTableModel {
        private static final char[] GREEK_CHARS = {'\u20ac', '\u0081', '\u201a', '\u0192', '\u201e', '\u2026', '\u2020', '\u2021', '\u02c6', '\u2030', '\u00b1', '\u0160', '\u0152', '\u008d', '\u008f', '\u0090', '\u2018', '\u2019', '\u201c', '\u201d', '\u00b7', '\u00b6', '\u017d', '\u2039'};
        private static final char[] MATH_SYMBOLS1 = {'\u00BB','\u02dc', '\u2014', '\u00b4', '\u00b8', '\u00a2', '\u203a', '\u00bd', '\u00d7', '\u00f7', '\u00a7', '\u00a9', '\u00ac', '\u00b0', '\u00be', '\u00bc', '\u00b3', '\u0153', '\u0161', '\u00de', '\u00af', '\u00bf', '\u0178', '\u017E', '{', '\u00AA'};
        private static final char[] MATH_SYMBOLS2 = {'\u00c6', '\u00b8', '\u007c', '\u0022', '\u003b', '\u003a', '\u02c6', '\\', '\u2013', '\u003c', '\u003e', '\u007b', '\u007d', '\u005b', '\u005d', '\u0028', '\u0029', '\u002c', '\u005e', '\u0152'};
        private static final char[] SPECIAL_SYMBOLS = {'\u0021', '\u0022', '\u0023', '\u0024', '\u0025', '\u0026', '\'', '\u003f', '\u0040', '\u005f', '\u00a1', '\u00a2', '\u00a3', '\u00a4', '\u00a5', '\u00a6', '\u00a8', '\u00ac', '\u00b9', '\u00d7', '\u00DE'};
        private final static int columnsCount = 5;
        private int dataLength = 0;
        private char[] charArray;

        /**
         * Returns the number of columns in this data table.
         *
         * @return the number of columns in the model
         */
        public final int getColumnCount() {
            return columnsCount;
        }

        /**
         * Returns the number of rows in this data table.
         *
         * @return the number of rows in the model
         */
        public final int getRowCount() {
            return (dataLength / columnsCount) + ((dataLength % columnsCount != 0) ? 1 : 0);    //call to super
        }

        public final boolean isCellEditable(final int row, final int column) {
            return false;
        }

        public final Object getValueAt(int row, final int column) {
            row = row * columnsCount + column;
            return (row < dataLength && row >= 0) ? new Character(charArray[row]) : null;
        }

        public final ArrayList getAvailableCharTables() {
            final ArrayList list = new ArrayList(5);
            list.add(AppPrefs.getLabel("characters.list.greek"));
            list.add(AppPrefs.getLabel("characters.list.math1"));
            list.add(AppPrefs.getLabel("characters.list.math2"));
            list.add(AppPrefs.getLabel("characters.list.specialSyms"));
            list.add(AppPrefs.getLabel("characters.list.allChars"));
            return list;
        }

        private final void setCharArray(final char[] array) {
            this.charArray = array;
            dataLength = array.length;
            fireTableDataChanged();
        }

        public final void selectCharTable(final int selectedIndex) {
            switch (selectedIndex) {
                case 0:
                    setCharArray(GREEK_CHARS);
                    break;
                case 1:
                    setCharArray(MATH_SYMBOLS1);
                    break;
                case 2:
                    setCharArray(MATH_SYMBOLS2);
                    break;
                case 3:
                    setCharArray(SPECIAL_SYMBOLS);
                    break;
                default:
                    setCharArray(RiderEditorKit.TI92PC_UNICODE_FONT);
            }
        }
    }

    private final static class LastUsedCharsTableModel extends AbstractTableModel {

        private final static int COLUMNS_COUNT = 8;
        private final Stack usedCharsList = new Stack();


        public final void addUsedChar(final Character ch) {
            if (!usedCharsList.contains(ch)) {
                usedCharsList.add(0, ch);
                usedCharsList.setSize(COLUMNS_COUNT);
                fireTableDataChanged();
            }
        }

        /**
         * Returns the number of columns in this data table.
         *
         * @return the number of columns in the model
         */
        public final int getColumnCount() {
            return COLUMNS_COUNT;
        }

        /**
         * Returns the number of rows in this data table.
         *
         * @return the number of rows in the model
         */
        public final int getRowCount() {
            return 1;    //call to super
        }

        public final boolean isCellEditable(final int row, final int column) {
            return false;
        }

        public final Object getValueAt(final int row, final int column) {
            return (column >= 0 && column < usedCharsList.size()) ? usedCharsList.get(column) : null;
        }
    }

    public boolean closeSoft() {
        return false;  //implement - call to super class
    }

    public void closeHard() {
        //implement - call to super class
    }

}