/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.ti89;

import net.wordrider.utilities.Utils;

import java.io.*;

/**
 * @author Vity
 */
public abstract class TIFileReader extends TIFile {
    //protected int dataAddress;

    public final boolean openFromFile(File file) throws InvalidDataTypeException, NotSupportedFileException, IOException {
        DataInputStream stream = null;
        try {
            if (!file.exists() && !file.createNewFile()) return false;
            stream = new DataInputStream(new FileInputStream(file));            
            readHeader(stream);
            readContent(stream);
            final char read = readChecksum(stream);
            final char get = getCheckSum();
            if (read != get) {
                Utils.log(Utils.WARN_LOG, "File '" + file.getPath() + "' was loaded but the file might be corrupted (CRC mismatch)");
                return false;
            }
            return true;
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    private static String getString(byte[] text) {
        final int size = text.length;
        int offset;
        for (offset = 0; offset < size; ++offset)
            if (text[offset] == 0)
                break;
        try {
            return new String(text, 0, offset, "ASCII");
        } catch (UnsupportedEncodingException e) {
            Utils.processException(e);
            return "unknown";
        }
    }

    private void readHeader(DataInputStream stream) throws NotSupportedFileException, InvalidDataTypeException, IOException {
        byte[] tempArray = new byte[8];
        stream.readFully(tempArray);
        final String temp = getString(tempArray);
        if (!isSupportedHeader(temp))
            throw new NotSupportedFileException(temp);
        stream.skip(2);//stream.readShort(); // 0x1 << 8
        stream.readFully(tempArray);
        fileInfo.setFolderName(getString(tempArray)); //writeString(stream,folderName,8);
        stream.readFully(tempArray = new byte[40]);
        fileInfo.setComment(getString(tempArray)); //writeString(stream,comment,40);
        stream.readShort(); //(0x1 << 8);//number of vars = 1
        stream.skip(4); //(0x52 << 24); //data address
        stream.readFully(tempArray = new byte[8]);
        fileInfo.setVarName(getString(tempArray)); // variable name //writeString(stream,varName,8);
        if (stream.readUnsignedByte() != getDataType())
            throw new InvalidDataTypeException();
        stream.skipBytes(1);
        stream.skipBytes(6);//readShort();//stream.writeShort(getDataType() << 8); //data type - text
        //stream.readInt();//stream.writeShort(0xFF << 8); //data type - text
        stream.readUnsignedByte();//stream.writeShort(0xFF << 8); //data type - text
        stream.readUnsignedByte();//stream.writeShort(0xFF << 8); //data type - text
        stream.skip(4); //4x zeroes
    }

    private static char readChecksum(DataInputStream stream) throws IOException {
        return (char) (stream.readUnsignedByte() + (stream.readUnsignedByte() << 8));
    }


    protected abstract void readContent(DataInputStream stream) throws IOException;

    protected abstract boolean isSupportedHeader(String header);

}