/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.utilities;

import net.wordrider.core.AppPrefs;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.net.URLClassLoader;

//import java.util.HashMap;

/**
 * @author Vity
 */
public final class Swinger {
    public static final int RESULT_CANCEL = 2;
    public static final int RESULT_NO = 1;
    public static final int RESULT_YES = 0;

    private static final String MESSAGE_ERROR_TITLE_CODE = "message.error.title";
    private static final String MESSAGE_CONFIRM_TITLE_CODE = "message.confirm.title";
    private static final String MESSAGE_INFORMATION_TITLE_CODE = "message.information.title";
    private static final String MESSAGE_BTN_YES_CODE = "message.button.yes";
    private static final String MESSAGE_BTN_NO_CODE = "message.button.no";
    private static final String MESSAGE_BTN_CANCEL_CODE = "message.button.cancel";

    private final static double FACTOR = 0.92;

//    private final static HashMap cachedResources = new HashMap(2);
    public static boolean ANTIALIASING = AppPrefs.getProperty("net.wordrider.area.antialiasing", false);

    public final static JRadioButton getRiderRadio(final String code) {
        JRadioButton radio = new JRadioButton(AppPrefs.getLabel(code));
        //checkbox.setMnemonic(AppPrefs.getMnemonic(code));
        return radio;
    }

    public final static JCheckBox getRiderCheckBox(final String code) {
        JCheckBox checkbox = new JCheckBox(AppPrefs.getLabel(code));
        checkbox.setMnemonic(AppPrefs.getMnemonic(code));
        return checkbox;
    }

    public final static JButton getRiderButton(final String code) {
        JButton button = new JButton(AppPrefs.getLabel(code));
        button.setMnemonic(AppPrefs.getMnemonic(code));
        return button;
    }

//    public final static ImageIcon getCachedIcon(final String fileName) {
//        if (cachedResources.containsKey(fileName)) {
//            return (ImageIcon) cachedResources.get(fileName);
//        } else {
//            ImageIcon icon = getIcon(fileName);
//            if (icon != null)
//                cachedResources.put(fileName, icon);
//            return icon;
//        }
//    }

    public final static ImageIcon getIcon(final String fileName) {
        if (fileName == null)
            return null;
        try {
            return new ImageIcon(ImageIO.read(((URLClassLoader) Swinger.class.getClassLoader()).findResource(Consts.IMAGESDIR + fileName)));
        } catch (final Exception e) {
            Utils.log(Utils.ERROR_LOG, "Cannot load image " + fileName + "\nReason:" + e.getMessage());
            return null;
        }
    }


    public static final class SelectAllOnFocusListener implements FocusListener {
        public void focusGained(final FocusEvent e) {
            if (!e.isTemporary()) {
                Component component = e.getComponent();
               ((JTextComponent) component).selectAll();
            }
        }

        public void focusLost(final FocusEvent e) {
        }
    }

    public static final void centerDialog(final Frame frame, final Component component) {
        final Dimension dlgSize = component.getPreferredSize();
        final Dimension frmSize = frame.getSize();
        final Point loc = frame.getLocation();
        component.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x,
                (frmSize.height - dlgSize.height) / 2 + loc.y);
    }


    public static final void inputFocus(final JTextComponent field) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                field.requestFocus();
            }
        });
    }

    public static final void showErrorDialog(final Frame frame, final String message) {
        JOptionPane.showMessageDialog(frame, message, AppPrefs.getLabel(Swinger.MESSAGE_ERROR_TITLE_CODE), JOptionPane.ERROR_MESSAGE);
    }

    public static final void showInformationDialog(final Frame frame, final String message) {
        JOptionPane.showMessageDialog(frame, message, AppPrefs.getLabel(Swinger.MESSAGE_INFORMATION_TITLE_CODE), JOptionPane.INFORMATION_MESSAGE);
    }


    public static final int getChoice(final Frame frame, final String message) {

        return JOptionPane.showOptionDialog(frame, message, AppPrefs.getLabel(MESSAGE_CONFIRM_TITLE_CODE),
                JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null, new String[]{AppPrefs.getLabel(MESSAGE_BTN_YES_CODE), AppPrefs.getLabel(MESSAGE_BTN_NO_CODE)},
                null);
    }

    public static int getChoiceCancel(final Frame frame, final String message) {
        return JOptionPane.showOptionDialog(frame, message, AppPrefs.getLabel(MESSAGE_CONFIRM_TITLE_CODE),
                JOptionPane.YES_NO_CANCEL_OPTION,
                JOptionPane.QUESTION_MESSAGE,
                null, new String[]{AppPrefs.getLabel(MESSAGE_BTN_YES_CODE), AppPrefs.getLabel(MESSAGE_BTN_NO_CODE), AppPrefs.getLabel(MESSAGE_BTN_CANCEL_CODE)},
                null);
    }

    public static Color brighter(final Color color) {
        if (color == null)
            return new JPanel().getBackground();
        int r = color.getRed();
        int g = color.getGreen();
        int b = color.getBlue();

        /* From 2D group:
         * 1. black.brighter() should return grey
         * 2. applying brighter to blue will always return blue, brighter
         * 3. non pure color (non zero rgb) will eventually return white
         */
        int i = (int) (1.0 / (1.0 - FACTOR));
        if (r == 0 && g == 0 && b == 0) {
            return new Color(i, i, i);
        }
        if (r > 0 && r < i) r = i;
        if (g > 0 && g < i) g = i;
        if (b > 0 && b < i) b = i;
        final Color result = new Color(Math.min((int) (r / FACTOR), 255),
                Math.min((int) (g / FACTOR), 255),
                Math.min((int) (b / FACTOR), 255));
        return result.equals(Color.WHITE) ? new JPanel().getBackground() : result;//new Color(0xD6D6D6) : result;
    }
}
